# Секция DISTINCT {#select-distinct}

Если указан `SELECT DISTINCT`, то в результате запроса останутся только уникальные строки. Таким образом, из всех наборов полностью совпадающих строк в результате останется только одна строка.

## Обработк NULL {#null-processing}

`DISTINCT` работает с [NULL](../../syntax.md#null-literal) как-будто `NULL` — обычное значение и `NULL==NULL`. Другими словами, в результате `DISTINCT`, различные комбинации с `NULL` встретятся только один раз. Это отличается от обработки `NULL` в большинстве других контекстов.

## Альтернативы {#alternatives}

Такой же результат можно получить, применив секцию [GROUP BY](group-by.md) для того же набора значений, которые указан в секции `SELECT`, без использования каких-либо агрегатных функций. Но есть от `GROUP BY` несколько отличий:

-   `DISTINCT` может применяться вместе с `GROUP BY`.
-   Когда секция [ORDER BY](order-by.md) опущена, а секция [LIMIT](limit.md) присутствует, запрос прекращает выполнение сразу после считывания необходимого количества различных строк.
-   Блоки данных выводятся по мере их обработки, не дожидаясь завершения выполнения всего запроса.

## Ограничения {#limitations}

`DISTINCT` не поддерживается, если `SELECT` имеет по крайней мере один столбец-массив.

## Примеры {#examples}

ClickHouse поддерживает использование секций `DISTINCT` и `ORDER BY` для разных столбцов в одном запросе. Секция `DISTINCT` выполняется до секции `ORDER BY`.

Таблица для примера:

``` text
┌─a─┬─b─┐
│ 2 │ 1 │
│ 1 │ 2 │
│ 3 │ 3 │
│ 2 │ 4 │
└───┴───┘
```

При выборе данных с помощью `SELECT DISTINCT a FROM t1 ORDER BY b ASC`, мы получаем следующий результат:

``` text
┌─a─┐
│ 2 │
│ 1 │
│ 3 │
└───┘
```

Если мы изменим направление сортировки `SELECT DISTINCT a FROM t1 ORDER BY b DESC`, мы получаем следующий результат:

``` text
┌─a─┐
│ 3 │
│ 1 │
│ 2 │
└───┘
```

Ряд `2, 4` был разрезан перед сортировкой.

Учитывайте эту специфику при разработке запросов.
